@@ -46,8 +46,7 @@ namespace rak {
 // Don't convert negative Timer to timeval and then back to Timer, that will bork.
 class timer {
  public:
-  timer() : m_time(0) {}
-  timer(int64_t usec) : m_time(usec) {}
+  timer(int64_t usec = 0) : m_time(usec) {}
   timer(timeval tv) : m_time((int64_t)(uint32_t)tv.tv_sec * 1000000 + (int64_t)(uint32_t)tv.tv_usec % 1000000) {}
 
   int32_t             seconds() const                    { return m_time / 1000000; }
@@ -63,8 +62,11 @@ class timer {
   }
 
   static timer        current();
+  static int64_t      current_seconds()                   { return current().seconds(); }
+  static int64_t      current_usec()                      { return current().usec(); }
+  static timer        from_minutes(uint32_t minutes)      { return rak::timer((uint64_t)minutes * 60 * 1000000); }
   static timer        from_seconds(uint32_t seconds)      { return rak::timer((uint64_t)seconds * 1000000); }
-  static timer        from_milliseconds(uint32_t seconds) { return rak::timer((uint64_t)seconds * 1000); }
+  static timer        from_milliseconds(uint32_t msec)    { return rak::timer((uint64_t)msec * 1000); }
 
   static timer        max()                              { return std::numeric_limits<int64_t>::max(); }
 
