@@ -51,6 +51,7 @@ public:
 
   using base_type::iterator;
   using base_type::reverse_iterator;
+  using base_type::const_iterator;
   using base_type::empty;
   using base_type::reserve;
 
@@ -83,7 +84,8 @@ public:
   data_type           at_memory(uint32_t offset, iterator part);
 
   // Check how much of the chunk is incore from pos.
-  uint32_t            incore_length(uint32_t pos);
+  bool                is_incore(uint32_t pos, uint32_t length = ~uint32_t());
+  uint32_t            incore_length(uint32_t pos, uint32_t length = ~uint32_t());
 
   bool                sync(int flags);
 
