@@ -42,12 +42,13 @@
 
 #include <rak/socket_address.h>
 
+#include "net/address_list.h"
+
 namespace torrent {
 
 class AvailableList : private std::vector<rak::socket_address> {
 public:
   typedef std::vector<rak::socket_address> base_type;
-  typedef std::list<rak::socket_address>   AddressList;
   typedef uint32_t                         size_type;
 
   using base_type::value_type;
@@ -58,6 +59,8 @@ public:
   using base_type::const_iterator;
   using base_type::reverse_iterator;
   using base_type::size;
+  using base_type::capacity;
+  using base_type::reserve;
   using base_type::empty;
   using base_type::clear;
 
@@ -76,6 +79,8 @@ public:
   size_type           max_size() const                   { return m_maxSize; }
   void                set_max_size(size_type s)          { m_maxSize = s; }
 
+  bool                want_more() const                  { return size() <= m_maxSize; }
+
   // This push is somewhat inefficient as it iterates through the
   // whole container to see if the address already exists.
   void                push_back(const rak::socket_address* sa);
