@@ -57,7 +57,8 @@ class ResourceManager;
 class PeerInfo;
 class ChunkManager;
 class ConnectionManager;
-class ThrottleManager;
+class Throttle;
+class DhtManager;
 
 typedef std::list<std::string> EncodingList;
 
@@ -75,14 +76,15 @@ public:
   ChunkManager*       chunk_manager()                           { return m_chunkManager; }
   ClientList*         client_list()                             { return m_clientList; }
   ConnectionManager*  connection_manager()                      { return m_connectionManager; }
+  DhtManager*         dht_manager()                             { return m_dhtManager; }
   
   Poll*               poll()                                    { return m_poll; }
   void                set_poll(Poll* p)                         { m_poll = p; }
 
   EncodingList*       encoding_list()                           { return &m_encodingList; }
 
-  ThrottleManager*    upload_throttle()                         { return m_uploadThrottle; }
-  ThrottleManager*    download_throttle()                       { return m_downloadThrottle; }
+  Throttle*           upload_throttle()                         { return m_uploadThrottle; }
+  Throttle*           download_throttle()                       { return m_downloadThrottle; }
 
   void                initialize_download(DownloadWrapper* d);
   void                cleanup_download(DownloadWrapper* d);
@@ -99,12 +101,13 @@ private:
   ChunkManager*       m_chunkManager;
   ClientList*         m_clientList;
   ConnectionManager*  m_connectionManager;
+  DhtManager*         m_dhtManager;
   Poll*               m_poll;
 
   EncodingList        m_encodingList;
 
-  ThrottleManager*    m_uploadThrottle;
-  ThrottleManager*    m_downloadThrottle;
+  Throttle*           m_uploadThrottle;
+  Throttle*           m_downloadThrottle;
 
   unsigned int        m_ticks;
   rak::priority_item  m_taskTick;
