@@ -1,5 +1,5 @@
 // libTorrent - BitTorrent library
-// Copyright (C) 2005-2006, Jari Sundell
+// Copyright (C) 2005-2007, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -69,11 +69,13 @@ public:
 
   bool                open_stream();
   bool                open_datagram();
+  bool                open_local();
   void                close();
 
   void                clear()                                 { m_fd = -1; }
 
   bool                bind(const rak::socket_address& sa);
+  bool                bind(const rak::socket_address& sa, unsigned int length);
   bool                connect(const rak::socket_address& sa);
 
   bool                listen(int size);
