@@ -61,7 +61,7 @@ SocketStream::read_stream_throws(void* buf, uint32_t length) {
   if (r == 0)
     throw close_connection();
 
-  if (r < 0)
+  if (r < 0) {
     if (rak::error_number::current().is_blocked_momentary())
       return 0;
     else if (rak::error_number::current().is_closed())
@@ -70,6 +70,7 @@ SocketStream::read_stream_throws(void* buf, uint32_t length) {
       throw blocked_connection();
     else
       throw connection_error(rak::error_number::current().value());
+  }
 
   return r;
 }
@@ -81,7 +82,7 @@ SocketStream::write_stream_throws(const void* buf, uint32_t length) {
   if (r == 0)
     throw close_connection();
 
-  if (r < 0)
+  if (r < 0) {
     if (rak::error_number::current().is_blocked_momentary())
       return 0;
     else if (rak::error_number::current().is_closed())
@@ -90,6 +91,7 @@ SocketStream::write_stream_throws(const void* buf, uint32_t length) {
       throw blocked_connection();
     else
       throw connection_error(rak::error_number::current().value());
+  }
 
   return r;
 }
