@@ -46,7 +46,6 @@
 #include "data/hash_queue.h"
 #include "data/hash_torrent.h"
 #include "download/available_list.h"
-#include "download/choke_manager.h"
 #include "download/chunk_selector.h"
 #include "download/chunk_statistics.h"
 #include "download/download_wrapper.h"
@@ -54,6 +53,7 @@
 #include "protocol/peer_factory.h"
 #include "peer/peer_info.h"
 #include "tracker/tracker_manager.h"
+#include "torrent/download/choke_queue.h"
 #include "torrent/download_info.h"
 #include "torrent/data/file.h"
 #include "torrent/peer/connection_list.h"
@@ -204,7 +204,10 @@ Download::is_hash_checking() const {
 
 void
 Download::set_pex_enabled(bool enabled) {
-  m_ptr->info()->change_flags(DownloadInfo::flag_pex_enabled, enabled);
+  if (enabled)
+    m_ptr->info()->set_pex_enabled();
+  else
+    m_ptr->info()->unset_flags(DownloadInfo::flag_pex_enabled);
 }
 
 Object*
@@ -374,7 +377,7 @@ Download::accepting_new_peers() const {
 
 uint32_t
 Download::uploads_max() const {
-  if (m_ptr->main()->upload_choke_manager()->max_unchoked() == ChokeManager::unlimited)
+  if (m_ptr->main()->upload_choke_manager()->max_unchoked() == choke_queue::unlimited)
     return 0;
 
   return m_ptr->main()->upload_choke_manager()->max_unchoked();
@@ -402,7 +405,7 @@ Download::set_uploads_max(uint32_t v) {
     throw input_error("Max uploads must be between 0 and 2^16."); 
 	 	 
   // For the moment, treat 0 as unlimited. 
-  m_ptr->main()->upload_choke_manager()->set_max_unchoked(v == 0 ? ChokeManager::unlimited : v); 
+  m_ptr->main()->upload_choke_manager()->set_max_unchoked(v == 0 ? choke_queue::unlimited : v); 
   m_ptr->main()->upload_choke_manager()->balance();
 }
 
@@ -437,6 +440,32 @@ Download::set_connection_type(ConnectionType t) {
   m_ptr->set_connection_type(t);
 }
 
+Download::HeuristicType
+Download::upload_choke_heuristic() const {
+  return (Download::HeuristicType)m_ptr->main()->upload_choke_manager()->heuristics();
+}
+
+void
+Download::set_upload_choke_heuristic(HeuristicType t) {
+  if ((choke_queue::heuristics_enum)t >= choke_queue::HEURISTICS_MAX_SIZE)
+    throw input_error("Invalid heuristics value.");
+  
+  m_ptr->main()->upload_choke_manager()->set_heuristics((choke_queue::heuristics_enum)t);
+}
+
+Download::HeuristicType
+Download::download_choke_heuristic() const {
+  return (Download::HeuristicType)m_ptr->main()->download_choke_manager()->heuristics();
+}
+
+void
+Download::set_download_choke_heuristic(HeuristicType t) {
+  if ((choke_queue::heuristics_enum)t >= choke_queue::HEURISTICS_MAX_SIZE)
+    throw input_error("Invalid heuristics value.");
+  
+  m_ptr->main()->download_choke_manager()->set_heuristics((choke_queue::heuristics_enum)t);
+}
+
 void
 Download::update_priorities() {
   m_ptr->receive_update_priorities();
@@ -452,4 +481,6 @@ Download::add_peer(const sockaddr* sa, int port) {
   m_ptr->main()->add_peer(sa_port);
 }
 
+DownloadMain* Download::main() { return m_ptr->main(); }
+
 }
