@@ -144,9 +144,19 @@ PollSelect::~PollSelect() {
   m_exceptSet->prepare();
 
   // Re-add this check when you've cleaned up the client shutdown procedure.
-  if (!m_readSet->empty() || !m_writeSet->empty() || !m_exceptSet->empty())
+  if (!m_readSet->empty() || !m_writeSet->empty() || !m_exceptSet->empty()) {
     throw internal_error("PollSelect::~PollSelect() called but the sets are not empty");
 
+    // for (SocketSet::const_iterator itr = m_readSet->begin(); itr != m_readSet->end(); itr++)
+    //   std::cout << "R" << (*itr)->file_descriptor() << std::endl;
+
+    // for (SocketSet::const_iterator itr = m_writeSet->begin(); itr != m_writeSet->end(); itr++)
+    //   std::cout << "W" << (*itr)->file_descriptor() << std::endl;
+
+    // for (SocketSet::const_iterator itr = m_exceptSet->begin(); itr != m_exceptSet->end(); itr++)
+    //   std::cout << "E" << (*itr)->file_descriptor() << std::endl;
+  }
+
 //   delete m_readSet;
 //   delete m_writeSet;
 //   delete m_exceptSet;
